﻿#include <iostream>
#include <vector>

using namespace std;

const size_t SCREEN_WIDTH = 80;
const size_t MAX_ASTERISK = SCREEN_WIDTH - 3 - 1;

struct Input {
    vector<double> numbers;
    size_t bin_count{};
};

Input input_data() {
    size_t number_count;
    cout << "Enter number count: ";
    cin >> number_count;

    Input in;
    in.numbers.resize(number_count);
    cout << "Enter " << number_count << " numbers: ";
    for (size_t i = 0; i < number_count; i++) {
        cin >> in.numbers[i];
    }

    cout << "Enter bin count: ";
    cin >> in.bin_count;

    return in;
}

void find_minmax(const vector<double>& numbers, double& min, double& max) {
    min = numbers[0];
    max = numbers[0];
    for (double x : numbers) {
        if (x < min)
            min = x;
        if (x > max)
            max = x;
    }
}

vector<size_t> make_histogram(const vector<double>& numbers, size_t bin_count) {
    vector<size_t> bins(bin_count, 0);
    double min, max;
    find_minmax(numbers, min, max);
    double bin_size = (max - min) / bin_count;

    for (size_t i = 0; i < numbers.size(); i++) {
        bool found = false;
        
        for (size_t j = 0; j < bin_count - 1 && !found; j++) {
            double lo = min + j * bin_size;
            double hi = min + (j + 1) * bin_size;
            if (numbers[i] >= lo && numbers[i] < hi) {
                bins[j]++;
                found = true;
            }
        }
        
        if (!found)
            bins[bin_count - 1]++;
    }
    return bins;
}

void show_histogram_text(const vector<size_t>& bins, size_t screen_width = SCREEN_WIDTH) {
    const size_t MAX_ASTERISK = screen_width - 3 - 1;
    size_t max_count = bins[0];
    for (size_t count : bins) {
        if (count > max_count)
            max_count = count;
    }
    for (size_t j = 0; j < bins.size(); j++) {
        if (bins[j] < 100)
            cout << " ";
        if (bins[j] < 10)
            cout << " ";
        cout << bins[j] << "|";

        size_t height = 0;
        if (max_count > MAX_ASTERISK) {
            height = static_cast<size_t>(static_cast<double>(bins[j]) / max_count * MAX_ASTERISK);
        }
        else {
            height = bins[j];
        }
        for (size_t i = 0; i < height; i++) {
            cout << "*";
        }
        cout << endl;
    }
}

int main()
{
    size_t number_count;
    cout << "Enter number count: ";
    cin >> number_count;
    vector<double> numbers(number_count);
    for (int i = 0; i < number_count; i++) {
        cin >> numbers[i];
    }
    size_t bin_count; //количество интервалов на которые мы разбиваем
    cout << "Enter bin count: ";
    cin >> bin_count;
    vector<size_t> bins(bin_count);
    double min = numbers[0];
    double max = numbers[0];
    for (double x : numbers) {
        if (x < min) {
            min = x;
        }
        else if (x > max) {
            max = x;
        }
    }
    double bin_size = (max - min) / bin_count;
    for (size_t i = 0; i < number_count; i++) {
        bool found = false;
        for (size_t j = 0; (j < bin_count - 1) && !found; j++) {
            auto lo = min + j * bin_size;
            auto hi = min + (j + 1) * bin_size;
            if ((lo <= numbers[i]) && (numbers[i] < hi)) {
                bins[j]++;
                found = true;
            }
        }
        if (!found) {
            bins[bin_count - 1]++;
        }
    }
    double max_count = bins[0];
    for (double x : bins) {
        if (x > max_count) {
            max_count = x;
        }
    }
    for (int j = 0; j < bin_count; j++) {
        if (bins[j] < 100) {
            cout << " ";
        }
        if (bins[j] < 10) {
            cout << " ";
        }
        cout << bins[j] << "|";

        size_t height;
        if (max_count > MAX_ASTERISK) {
            size_t count = bins[j];
            height = 76 * (static_cast<double>(count) / max_count);
        }
        else {
            height = bins[j];
        }

        for (size_t i = 0; i < height; i++) {
            cout << "*";
        }
        cout << endl;
    }
}